;=================================================
; Layer 2 Smash: Savage Edition
; By Erik
;=================================================
; This is an version of the Layer 2 smasher which
; stuns the player in place if he's on the ground
; once it hits.
; It's very customizable.
;=================================================


; Defines and tables

!FreeRAM      =      $18CB                ; 2 bytes. must be between $0100 and $1fff

!SmashPos     =      $0040                ; where will the smasher stop before rising again.

timeBefFalling:                           ; time until the smasher falls.
       dw $0020,$0010,$0010,$0020,$0030   ; smaller the value, smaller the delay

timeBefOffset:                            ; time before the smasher lowers by one tile.
       dw $0060,$0020,$0020,$0040,$0060   ; smaller the value, smaller the delay

smashSpeed:                               ; speed of the smasher when it falls.
       dw $0004,$0001,$0002,$0002,$0005   ; the smaller the value, the slower it smashes

timeBefRising:                            ; time before the smasher goes back.
       dw $0030,$002C,$0028,$0024,$0020   ; smaller the value, smaller the delay

riseSpeed:                                ; speed of the smasher when it rises.
       dw $0002,$0001,$0001,$0002,$0004   ; the smaller the value, the slower it smashes

timeToFreeze:
       dw $0040,$0060,$0060,$0060,$0060

screenStart:
minScreenPos:                             ; screens with activity and no activity.
       dw $0000,$FFFF,$FFFF,$FFFF,$FFFF

maxScreenPos:
       dw $0200,$FFFF,$FFFF,$FFFF,$FFFF
screenEnd:

!FreeRAM      :=     !FreeRAM|!addr

init:
       LDA $71              ;\
       CMP #$0A             ; | don't run on castle intros
       BEQ .return          ;/
       LDA #$C0             ;\  initial layer 2 pos
       STA $1468|!addr      ;/
       ; STZ $1412|!addr    ;   no horizontal scroll
       LDY #$00             ;\
       LDA timeBefFalling,y ; | init timer
       STA !FreeRAM         ;/
       STZ !FreeRAM+1
.return
       RTL

main:
       LDA $71                            ;\
       CMP #$0A                           ; | don't run on castle intros
       BEQ .return                        ;/
       LDA $9D                            ;\
       ORA $13D4|!addr                    ; | don't run on game locked or pause
       BNE .return                        ;/
       REP #$20                           ;   16-bit A
       LDY.b #screenEnd-screenStart/2-2   ;   number of screens to check
.posLoop
       LDA $1466|!addr                    ;\
       CMP minScreenPos,y                 ; |
       BCC .checkNextScreen               ; | check if mario is in the specified positions
       CMP maxScreenPos,y                 ; |
       BCS .checkNextScreen               ;/
       SEP #$20                           ;   8-bit A
       LDA.w !FreeRAM                     ;\  check if time for move is now
       BEQ smash                          ;/
       DEC.w !FreeRAM                     ;   if not, decrement timer and return
       RTL

.checkNextScreen
       DEY                                ;\
       DEY                                ; | check next set of screens
       BPL .posLoop                       ;/
       SEP #$20                           ;   8-bit A
       STZ $1468|!addr                    ;   reset layer 2 pos
.return
       RTL

smash:
       lda $1337
       LDA !FreeRAM+1       ;\  check if fall or rise
       BEQ .offset          ;/
       CMP #$01
       BEQ .fall
.rise
       REP #$20
       LDA $1468|!addr      ;\
       CLC                  ; | increase layer 2 pos
       ADC riseSpeed,y      ;/
       CMP #$00C0           ;\  return if not time to offset
       BNE +                ;/
       SEP #$20
       STZ !FreeRAM+1       ;   decrease state
       LDA timeBefOffset,y  ;\  set time before the smasher offsets
       STA !FreeRAM         ;/
       REP #$20
       LDA #$00C0           ;\  set layer 2 position
+      STA $1468|!addr      ;/
       SEP #$20
..return
       RTL
.fall
       REP #$20
       LDA $1468|!addr      ;\
       SEC                  ; | decrease layer 2 pos
       SBC smashSpeed,y     ;/
       CMP #!SmashPos       ;\  return if not time to return (...)
       BNE ++               ;/
       SEP #$20
       INC !FreeRAM+1       ;   increase state
       LDA #$60             ;\  shake layer 1
       STA $1887|!addr      ;/
       LDA $77
       AND #$04
       BEQ +

       LDA timeToFreeze,y
       STA $18BD|!addr
       LDA #$0A
       JSR DrawSmoke
       LDA #$FC
       JSR DrawSmoke

+      LDA #$18             ;\  play SFX
       STA $1DFC|!addr      ;/
       LDA timeBefRising,y  ;\  set time before the smasher rises
       STA !FreeRAM         ;/
       REP #$20
       LDA.w #!SmashPos     ;\  set layer 2 position
++     STA $1468|!addr      ;/
       SEP #$20
..return
       RTL

.offset
       LDA $1468|!addr      ;\
       DEC                  ; | decrease layer 2 pos
       STA $1468|!addr      ;/
       CMP #$B0             ;\  return if not time to fall
       BNE ..return         ;/
       INC !FreeRAM+1       ;   increase state
       LDA timeBefFalling,y ;\  set time before the smasher falls
       STA !FreeRAM         ;/
..return
       RTL

DrawSmoke:
       XBA
       LDY #$03

-      LDA $17C0|!addr,y
       BEQ +
       DEY
       BPL -
       RTS

+      LDA #$03
       STA $17C0|!addr,y
       LDA #$14     
       STA $17CC|!addr,y
       LDA $96
       CLC
       ADC #$1C
       STA $17C4|!addr,y
       XBA
       CLC
       ADC $94
       STA $17C8|!addr,y
       RTS
